import {
  ConflictException,
  Injectable,
  NotFoundException,
} from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { FindOptionsWhere, MoreThan, Repository } from 'typeorm';
import { TipoUsuario } from './entity/tipo-usuario.entity';

@Injectable()
export class TipoUsuarioService {
  constructor(
    @InjectRepository(TipoUsuario) private repository: Repository<TipoUsuario>,
  ) {}

  create(tipo_usuario: string): Promise<{ message: string }> {
    // Buscamos un registro con ese nombre
    return this.repository
      .findOne({ where: { tipo_usuario } })
      .then((existeTipoUsuario) => {
        // Sacamos error si existe
        if (existeTipoUsuario)
          throw new ConflictException('Ya existe este tipo de usuario');
        // Creamos y guardamos un registro
        return this.repository.save(this.repository.create({ tipo_usuario }));
      })
      .then((_) => ({
        message: 'Se creó correctamente un nuevo tipo de usuario.',
      }));
  }

  crearTipoUsuario(id_tipo_usuario: number): TipoUsuario {
    return this.repository.create({ id_tipo_usuario });
  }

  findAll(informacion?: string): Promise<TipoUsuario[]> {
    const busqueda: FindOptionsWhere<TipoUsuario> = {
      id_tipo_usuario: MoreThan(2),
    };

    if (informacion && informacion === 'operador')
      busqueda.id_tipo_usuario = MoreThan(4);
    return this.repository.find({
      where: busqueda,
      order: { tipo_usuario: 'ASC' },
    });
  }

  findById(id_tipo_usuario: number): Promise<TipoUsuario> {
    return this.repository
      .findOne({ where: { id_tipo_usuario } })
      .then((tipoUsuario) => {
        if (!tipoUsuario)
          throw new NotFoundException('No existe este id tipo usuario.');
        return tipoUsuario;
      });
  }

  findByTipoUsuario(tipo_usuario: string): Promise<TipoUsuario> {
    return this.repository.findOne({ where: { tipo_usuario } });
  }
}
